//////////////////////////////////////////////////
//////////////////Tree Generator//////////////////
//////////////////////////////////////////////////
////11.06.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

var TG_WindStrength = 5;

//creates a new tree part given by Type on the Parent entity with the given action
ENTITY* TG_CreatePart(ENTITY* Parent, int Type, long Act)
{
	//get the parents filename
	STRING* TempString = str_create("#40");
	str_for_entfile(TempString,Parent);
	
	VECTOR temp;
	
	//do Type specific things
	switch(Type)
	{
		case 0:
			//get a random position on Parent
			vec_set(temp,vector(Parent.x,Parent.y,Parent.max_z));
			vec_lerp(temp,Parent.x,temp,my.skill11+random(1.0-Parent.skill11));
			//set the filename
			str_trunc(TempString,4);
			str_cat(TempString,"br.mdl");
			break;
		
		case 1:
			//get a random position on Parent
			vec_for_vertex(temp,Parent,Parent.skill6);
			vec_lerp(temp,Parent.x,temp,Parent.skill18+random(1.0-Parent.skill18));
			//set the filename
			str_trunc(TempString,6);
			str_cat(TempString,"tw.mdl");
			break;
		
		case 2:
			//get the position of the ending of Parent
			if(Parent.skill1 == 0)
			{
				vec_for_vertex(temp,Parent,Parent.skill6);
			}else
			{
				vec_for_vertex(temp,Parent,Parent.skill13);
			}
			//set the filename
			str_trunc(TempString,6);
			str_cat(TempString,"lf.tga");
			break;
		
		default:
			return 0;
	}

	//create the new entity
	ENTITY* Ent = ent_create(TempString,temp,Act);
	
	//copy Parents skills to the ones of the new created entity
	int i;
	for(i = 0; i < 100;i++)
 	{
 		Ent.skill[i] = Parent.skill[i];
 	}
	
	//copy the flags
	Ent.flags = Parent.flags;
	
	//set the Type
	Ent.skill1 = Type;
	
	//make the new entity passable
	set(Ent,PASSABLE);
	
	//return the pointer to the new entity
	return Ent;
}

//alphatestmaterial for the leaves
MATERIAL* TG_Leaf_mat =
{
	effect =
	"
		//variables
		float4x4 matProj, matView, matWorld, matWorldViewProj;
		float4 vecViewPos, vecFog, vecLight, vecSunDir, vecSunColor;
		
		//the texture sampler
		sampler grassSampler;
		
		//Vertexshader
		void vs_grass(	in float4 inPos : POSITION,
							in float2 inTex0 : TEXCOORD0,
							out float4 Pos	: POSITION,
							out float  Fog	: FOG,
							out float2 Tex0 : TEXCOORD1)
		{
			inPos = mul(inPos,matWorld);
			
			//do the final vertex transformation
			Pos = mul(float4(inPos.xyz,1.0),mul(matView,matProj));
			
			//fog
			Fog = 1 - (distance(inPos,vecViewPos)-vecFog.x) * vecFog.z;
			
			//pass the texcoords
			Tex0 = inTex0;
		}
		
		//the pixelshader sets the transparency to 1 or 0
		float4 ps_grass(float2 Tex0 : TEXCOORD1) : COLOR 
		{	
			//get the pixels color
			float4 Color = tex2D(grassSampler,Tex0);
			
			//return the color
			return float4(Color.rgb*(vecLight.rgb+0.5),step(0.5,Color.a));
		}
		
		// Techniques
		technique grass 
		{
			pass p0 
			{
				//set the render states
				zWriteEnable = true;
				alphaTestEnable = true;
				
				//use the shader
				VertexShader = compile vs_1_1 vs_grass();
				PixelShader = compile ps_2_0 ps_grass();
			}
		}
	";
}

//function for the animated leaves
void TG_Leaf()
{
	wait(1);
	
	//set material and scale
	my.material = TG_Leaf_mat;
	vec_scale(my.scale_x,my.skill19-my.skill20*0.5+random(my.skill20));
	
	//animate the leaf
	while(is(my,FLAG1))
	{
		//set rotation with animation
		vec_set(my.pan,camera.pan);
		ang_add(my.pan,vector(sin((total_ticks+my.x)*TG_WindStrength)*TG_WindStrength*2,cos((total_ticks+my.y)*TG_WindStrength)*TG_WindStrength*2,0));
		
		wait(1);
	}
}

//function for the twigs
void TG_Twig()
{
	wait(1);
	
	//set position and scale
	ang_rotate(my.pan,vector(random(360),my.skill16-my.skill17*0.5+random(my.skill17),0));
	vec_scale(my.scale_x,my.skill14-my.skill15*0.5+random(my.skill15));
	
	//create a leaf
	TG_CreatePart(my,2,TG_Leaf);
	
	//remove the twig if wished
	if(is(my,FLAG2))
	{
		ent_remove(me);
	}
}

//function for the branchs
void TG_Branch()
{
	wait(1);
	
	//set position and scale
	ang_rotate(my.pan,vector(random(360),my.skill9-my.skill10*0.5+random(my.skill10),0));
	vec_scale(my.scale_x,my.skill7-my.skill8*0.5+random(my.skill8));
	
	//create a leaf
	TG_CreatePart(my,2,TG_Leaf);

	//create twigs
	while(my.skill12 > 0)
	{
		TG_CreatePart(my,1,TG_Twig);
		my.skill12 -= 1;
	}
}


//title: Tree Generator
//section: Trunk
//skill1: Size_X 1
//skill2: Size_Y 1
//skill3: Size_Z 1
//skill4: Randomize 0.5
//section:Branches
//skill5: Number 5
//skill6: EndVertex 26
//skill7: Size 2
//skill8: Randomize 1
//skill9: Angle 80
//skill10: Randomize 10
//skill11: Height 0.3
//section: Twigs
//skill12: Number 5
//skill13: EndVertex 26
//skill14: Size 0.75
//skill15: Randomize 0.3
//skill16: Angle 90
//skill17: Randomize 1
//skill18: Dist 0.3
//flag2: RemoveTwigs 1
//section: Leaves
//skill19: Size 1
//skill20: Randomize 1
//flag1: WindAnimation 1
action TG_Main()
{
	set(my,POLYGON);
	
	//set a random scale
	vec_set(	my.scale_x,
				vector(	my.skill1-(my.skill4/2)+random(my.skill4),
							my.skill2-(my.skill4/2)+random(my.skill4),
							my.skill3-(my.skill4/2)+random(my.skill4)
						)
			 );
	
	//set the new bounding box
	c_setminmax(my);
	
	//set a random angle
	my.pan = random(360);
	
	//create the branches
	while(my.skill5 > 0)
	{
		TG_CreatePart(my,0,TG_Branch);
		my.skill5 -= 1;
	}
}